#!/usr/bin/env python
# setup.py generated by flit for tools that don't yet use PEP 517

from distutils.core import setup

packages = \
['kisee', 'kisee.providers']

package_data = \
{'': ['*']}

install_requires = \
['aiohttp',
 'aiomysql',
 'asyncpg',
 'bcrypt',
 'coreapi',
 'cryptography',
 'psutil',
 'pyjwt',
 'sentry-sdk',
 'shortuuid',
 'toml']

extras_require = \
{'dev': ['hypothesis',
         'pycalver',
         'pytest',
         'pytest-cov',
         'pytest-aiohttp',
         'flake8',
         'pylint',
         'black',
         'bandit',
         'mypy']}

entry_points = \
{'console_scripts': ['kisee = kisee.kisee:main',
                     'kisee-quickstart = kisee.quickstart:main']}

setup(name='kisee',
      version='19.11.0',
      description='HTTP server managing identities.',
      author='Julien Palard',
      author_email='julien@python.org',
      url='https://github.com/meltygroup/kisee',
      packages=packages,
      package_data=package_data,
      install_requires=install_requires,
      extras_require=extras_require,
      entry_points=entry_points,
      python_requires='>=3.6',
     )
